/*
 *	CHAOS target for iptables
 *	Copyright © Jan Engelhardt <jengelh@computergmbh.de>, 2006 - 2007
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License; either version
 *	2 or 3 as published by the Free Software Foundation.
 */
#include <getopt.h>
#include <stdio.h>
#include <string.h>

#include <xtables.h>
#include <linux/netfilter/x_tables.h>
#include <linux/netfilter/xt_CHAOS.h>

static void CHAOS_help(void)
{
	printf(
		"CHAOS target v%s options:\n"
		"  --delude     Enable DELUDE processing for TCP\n"
		"  --tarpit     Enable TARPIT processing for TCP\n",
		IPTABLES_VERSION);
	return;
}

static int CHAOS_parse(int c, char **argv, int invert, unsigned int *flags,
    const void *entry, struct xt_entry_target **target)
{
	struct xt_chaos_info *info = (void *)((*target)->data);
	switch(c) {
		case 'd':
			info->variant = XTCHAOS_DELUDE;
			*flags |= 0x02;
			return 1;
		case 't':
			info->variant = XTCHAOS_TARPIT;
			*flags |= 0x01;
			return 1;
	}
	return 0;
}

static void CHAOS_check(unsigned int flags)
{
	if(flags != 0x03)
		return;
	/* If flags == 0x03, both were specified, which should not be. */
	exit_error(PARAMETER_PROBLEM,
	           "CHAOS: only one of --tarpit or --delude may be specified");
	return;
}

static void CHAOS_print(const void *ip, const struct xt_entry_target *target,
    int numeric)
{
	const struct xt_chaos_info *info = (const void *)target->data;
	switch(info->variant) {
		case XTCHAOS_DELUDE:
			printf("DELUDE ");
			break;
		case XTCHAOS_TARPIT:
			printf("TARPIT ");
			break;
		default:
			break;
	}
	return;
}

static void CHAOS_save(const void *ip,
    const struct xt_entry_target *target)
{
	const struct xt_chaos_info *info = (const void *)target->data;
	switch(info->variant) {
		case XTCHAOS_DELUDE:
			printf("--delude ");
			break;
		case XTCHAOS_TARPIT:
			printf("--tarpit ");
			break;
		default:
			break;
	}
	return;
}

static const struct option CHAOS_opts[] = {
	{"delude", 0, NULL, 'd'},
	{"tarpit", 0, NULL, 't'},
	{NULL},
};

static struct xtables_target CHAOS_reg = {
	.name          = "CHAOS",
	.version       = IPTABLES_VERSION,
	.family        = AF_INET,
	.size          = XT_ALIGN(sizeof(struct xt_chaos_info)),
	.userspacesize = XT_ALIGN(sizeof(struct xt_chaos_info)),
	.help          = CHAOS_help,
	.parse         = CHAOS_parse,
	.final_check   = CHAOS_check,
	.print         = CHAOS_print,
	.save          = CHAOS_save,
	.extra_opts    = CHAOS_opts,
};

void _init(void)
{
	xtables_register_target(&CHAOS_reg);
	return;
}
